package org.btcollider.cnc.comm;

import java.io.File;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import org.btcollider.cnc.keysrv.KeyServer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CommServer {
  final Logger log = LoggerFactory.getLogger(CommServer.class);

  private int port;
  private ServerSocket serverSocket;
  private KeyServer keyServer;
  private String succFile;
  private String dumpFile;

  public CommServer(int port, KeyServer keyServer, String succFile, String dumpFile) {
    this.port = port;
    this.keyServer = keyServer;


    try {
      (new File(succFile)).createNewFile();
    } catch (IOException e1) {
      log.error("Error while trying to create success file at: {}", e1);
    }
    try {
      (new File(dumpFile)).createNewFile();
    } catch (IOException e1) {
      log.error("Error while trying to create dump file at: {}", e1);
    }

    if (!Files.isWritable(Paths.get(succFile))) {
      log.warn("Success file path is not writeable: {}", succFile);
      try {
        this.succFile = Files.createTempFile("btcollider", ".succ").toFile().getAbsolutePath();
      } catch (IOException e) {
        log.error("Couldn't create temporary success file", e);
      }
      log.warn("Temporary success file created. Path is: {}", this.succFile);
    } else {
      this.succFile = succFile;
      log.info("Success file path is: {}", this.succFile);
    }

    if (!Files.isWritable(Paths.get(dumpFile))) {
      log.warn("Dump file path is not writeable: {}", dumpFile);
      try {
        this.dumpFile = Files.createTempFile("btcollider", ".dump").toFile().getAbsolutePath();
      } catch (IOException e) {
        log.error("Couldn't create temporary dump file", e);
      }
      log.warn("Temporary dump file created. Path is: {}", this.dumpFile);
    } else {
      this.dumpFile = dumpFile;
      log.info("Dump file path is: {}", this.dumpFile);
    }
  }

  public void listen() throws CommException {
    if (serverSocket == null || serverSocket.isClosed()) {
      openSocket();
    }

    ExecutorService es = Executors.newCachedThreadPool();

    while (true) { // NOSONAR
      try {
        Socket socket = serverSocket.accept();
        log.debug("Got connection from {}", socket.getInetAddress());
        es.execute(new ClientWorker(socket, keyServer, succFile, dumpFile));

      } catch (IOException e) {
        log.error("Couldn't establish connections on socket", e);
        throw new CommException(e);
      }
    }
  }

  public void stop() {
    closeSocket();
  }

  private void openSocket() throws CommException {
    log.debug("Opening socket on port {}", port);

    try {
      this.serverSocket = new ServerSocket(port);
    } catch (IOException e) {
      log.error("Couldn't allocate server socket", e);
      throw new CommException(e);
    }
  }

  private void closeSocket() {
    if (this.serverSocket != null && !this.serverSocket.isClosed()) {
      try {
        serverSocket.close();
      } catch (IOException e) {
        log.error("Couldn't close server socket cleanly", e);
      }
    }
  }
}
