package org.btcollider.cnc.keysrv;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import org.btcollider.cnc.dto.KeyRange;

public interface KeyServer {
  /**
   * Gets a free (= not setInWork and not setSearched) KeyRange from the pool of possible key ranges
   *
   * @return A free KeyRange or null if none such exists
   */
  KeyRange getRange();

  /**
   * Sets a KeyRange as being currently worked on.
   *
   * As long as a KeyRange is not setSearched, there is no guarantee that an inWork KeyRange will be
   * finished in any specific time frame. The KeyServer has to clean up dangling inWorks in good
   * faith. Unexpected setSearched of already cleaned-up inWorks must not be of any harm.
   *
   * @param keyRange The KeyRange to be set as being worked on, not null
   */
  void setInWork(KeyRange keyRange);

  /**
   * Sets a KeyRange as being searched, finalizing this range A searched KeyRange need not be
   * processed any more
   *
   * @param keyRange The KeyRange to be set as searched, not null
   */
  void setSearched(KeyRange keyRange);

  /**
   * Dumps the state of the keys managed by this KeyServer to a file. A KeyServer must be able to
   * read back the status of its keys from its dump file, but not necessarily from other KeyServers.
   *
   * @param out The output stream to write the dump to. Not null.
   */
  void dump(OutputStream out) throws IOException;

  /**
   * Recreates the keys of a KeyServer from a dump. A KeyServer must be able to read back the status
   * of its keys form a dump file, but not necessarily from other KeyServers.
   *
   * Use this method only directly after initialization on a fresh KeyServer. Otherwise the state
   * might be unexpected.
   *
   * @param int The input stream to read the dump from. Not null.
   * @throws IOException The undumping failed. Keys are in an undefined state.
   */
  void undump(InputStream in) throws IOException;

}
